/*
 * Author: Dušan Kolář
 */

#ifndef PARSE_GOOSE
#define PARSE_GOOSE

typedef enum e_goose_parse_error {
    GPE_OK,
    GPE_TOO_SHORT,                  // passed buffer is shorter then smallest packet
    GPE_LENGTH_MISMATCH,            // lengths inside don't match
    GPE_OUT_OF_DATA,                // running out of the passed data length
    GPE_TLV_LARGE,                  // size of TLV goes more than 0xffff
    GPE_TLV_INDEFINITE,             // TLV length set as indefinite
    
    GPE_GOOSE_MAGIC_FAIL,           // GOOSE magic expected, not found
    GPE_APDU_MAGIC_FAIL,            // APDU magic expected, not found
    GPE_GOCB_REF_MAGIC_FAIL,        // gocbRef magic expected, not found
    GPE_TTL_MAGIC_FAIL,             // Time to live magic expected, not found
    GPE_DATA_SET_MAGIC_FAIL,        // Data set magic expected, not found
    GPE_GOOSE_ID_MAGIC_FAIL,        // GOOSE ID magic expected, not found
    GPE_T_MAGIC_FAIL,               // T magic expected, not found
    GPE_STATUS_NUM_MAGIC_FAIL,      // Status number magic expected, not found
} goose_parse_error_t;


#include <inttypes.h>


typedef struct s_goose_data {
    uint16_t     appid;             // APPID
    uint16_t     gocb_ref_len;      // gocbRef (GOOSE ctrl block) data length
    uint8_t     *gocb_ref_data;     // pointer to start of gocbRef (GOOSE ctrl block) data
    uint16_t     data_set_len;      // Data set length
    uint8_t     *data_set_data;     // pointer to start of data set
    uint16_t     goose_id_len;      // GOOSE ID length
    uint8_t     *goose_id_data;     // pointer to start of GOOSE ID
    uint16_t     status_num_len;    // Status number (changes) length
    uint8_t     *status_num_data;   // pointer to status number
    uint32_t     status_num_value;  // Status number value;
} goose_data_t;

goose_data_t *parse_goose(uint8_t *input, uint32_t size, goose_parse_error_t *error);

#endif
